//------------------------------------------------------------------------
// Project		: OHM_LCMXO2_4000HC BREAKOUT BOARD DEMO
// Module Name	: OHM_LCMXO2_4000HC
// Author		: Dan Neasmith
// Created		: 09/20/2019
//------------------------------------------------------------------------
// Description 	: A simple Module driving an LED array by sending UART comands over Bluetooth
//------------------------------------------------------------------------
// Modification history : 
//------------------------------------------------------------------------
module OHM_LCMXO2_4000HC_TOP (
	input 			CLK,
	input 			RST_n,
	input 			PC_UART_TX,
	input			   BT_UART_TX,
	output 			PC_UART_RX,
	output			BT_UART_RX,
	output 			BT_RST,
	output [9:0]	LED
	);
	
	//-------------------------------------------------------------------------------
	// Parameters
	//-------------------------------------------------------------------------------
	
	localparam 		MAX_MSG_LEN = 30;
	
	//-------------------------------------------------------------------------------
	// Reset Distribution
	//-------------------------------------------------------------------------------
	
	wire 			resetn;
	//wire			bugreset = 1;
	assign 			resetn = RST_n;
	assign 			BT_RST = resetn;
	
	//-------------------------------------------------------------------------------
	// Wires and registers
	//-------------------------------------------------------------------------------
	
	//Clock division
	wire 			clk_pwm;
	wire 			clk_1Hz;
	
	//Uart RX Module
	wire 			rx_serial;
	wire 			rx_dv;
	wire [7:0] 		rx_byte;
	
	//UART TX Module
	reg 			tx_dv;
	reg [7:0] 		tx_byte;
	wire 			tx_active;
	wire 			tx_serial;
	wire 			tx_done;
	
	//Uart Data 
	reg 			uart_done_rx;
	reg [7:0] 		uart_8bit_rx;
	//reg [7:0] 		uart_8bit_tx;
	
	reg [8:0]		LED_Binary;
	
	reg [3:0]		state = 1;
	reg [3:0]		next = 1;
	reg [3:0]		count = 0;
	//reg 			check = 0;
	
	//reg [7:0]		test_8bit = ">";
	
	//-------------------------------------------------------------------------------
	// Assignments
	//-------------------------------------------------------------------------------
	
	assign 			rx_serial = BT_UART_TX;
	assign			BT_UART_RX = tx_serial;
	assign			PC_UART_RX = rx_serial;
	//assign			LED[4] = rx_serial;
	
	assign 			BT_UART_RX = tx_serial;
	
	assign			LED[4] = clk_1Hz;
	
	assign 			LED[9] = ~LED_Binary[0];
	assign 			LED[8] = ~LED_Binary[1];
	assign 			LED[7] = ~LED_Binary[2];
	assign 			LED[5] = ~LED_Binary[3];
	assign 			LED[6] = ~LED_Binary[4];
	assign 			LED[2] = ~LED_Binary[5];
	assign 			LED[3] = ~LED_Binary[6];
	assign 			LED[1] = ~LED_Binary[7];
	assign 			LED[0] = ~LED_Binary[8];
	
	//-------------------------------------------------------------------------------
	// Clock division
	//-------------------------------------------------------------------------------
	
	CLK_DIV # (.divider(1000)) pwm_div //1 kHz for pwn
	(
		.rst_n		(RST_n),
		.clk		(CLK),
		.clk_o		(clk_pwm)
	);
	
	CLK_DIV # (.divider(10000000)) hert_div //1 Hz
	(
		.rst_n		(RST_n),
		.clk		(CLK),
		.clk_o		(clk_1Hz)
	);
	
	//-------------------------------------------------------------------------------
	// State Machine
	//-------------------------------------------------------------------------------
	
	always @(posedge CLK or negedge RST_n) begin
		if(!RST_n) begin
			state <= 0;
		end
		else begin
			state <= next;
		end
	end
	
	//-------------------------------------------------------------------------------
	// UART RX and TX
	//-------------------------------------------------------------------------------
	
	always @ (posedge CLK or negedge RST_n) begin
		if (!RST_n) begin
			uart_done_rx <= 0;
			uart_8bit_rx <= 0;
		end
		else begin
			uart_done_rx <= rx_dv;
			uart_8bit_rx <= rx_byte;
		end
	end
	
	always @ (posedge CLK or negedge RST_n) begin
		if(!RST_n) begin
			tx_dv <= 0;
			tx_byte <= 0;
		end
		else if(uart_done_rx) begin
			tx_dv <= 1;
			tx_byte <= uart_8bit_rx;
		end
		else if(tx_done) begin
			tx_dv <= 0;
			tx_byte <= 0;
		end
	end
	
	UART_RX # (.CLKS_PER_BIT(87)) uart_rx
	(
		.i_Clock	(CLK),
		.i_Rx_Serial(rx_serial),
		.o_Rx_DV	(rx_dv),
		.o_Rx_Byte	(rx_byte)
	);
	
	UART_TX # (.CLKS_PER_BIT(87)) uart_tx
	(
		.i_Clock	(CLK),
		.i_Tx_DV	(tx_dv),
		.i_Tx_Byte	(tx_byte),
		.o_Tx_Active(tx_active),
		.o_Tx_Serial(tx_serial),
		.o_Tx_Done	(tx_done)
	);
	
	//-------------------------------------------------------------------------------
	// UART Handling
	//-------------------------------------------------------------------------------
	
	
	
	//-------------------------------------------------------------------------------
	// LED Drivers
	//-------------------------------------------------------------------------------
	
	always @(*) begin
		//LED_Binary = 8'b00000000;
		case(uart_8bit_rx)
			"0" : begin next <= 0; end
			"1" : begin next <= 1; end
			"2" : begin next <= 2; end
			"3" : begin next <= 3; end
			"4" : begin next <= 4; end
			"5" : begin next <= 5; end
			"6" : begin next <= 6; end
			"7" : begin next <= 7; end
			"8" : begin next <= 8; end
			"9" : begin next <= 9; end
			"A" : begin next <= 10; end
			"B" : begin next <= 11; end
			"C" : begin next <= 12; end
			"D" : begin next <= 13; end
			">" : begin next <= 14; end
			
			default : begin next <= 0; end
		endcase
	end
	
	always @(posedge clk_1Hz) begin
		case(state) 
		
			0 : 
				begin 
					LED_Binary <= 9'b000000000; 
					count = 0;
				end
				
			1 : 
				begin 
					LED_Binary <= 9'b000000001; 
					count = 0;
				end
				
			2 : 
				begin 
					LED_Binary <= 9'b000000010; 
					count = 0;
				end
				
			3 : 
				begin 
					LED_Binary <= 9'b000000100; 
					count = 0;
				end
				
			4 : 
				begin 
					LED_Binary <= 9'b000001000;
					count = 0;
				end
				
			5 : 
				begin 
					LED_Binary <= 9'b000010000; 
					count = 0;
				end
				
			6 : 
				begin 
					LED_Binary <= 9'b000100000; 
					count = 0;
				end
				
			7 : 
				begin 
					LED_Binary <= 9'b001000000; 
					count = 0;
				end
				
			8 : 
				begin 
					LED_Binary <= 9'b010000000; 
					count = 0;
				end
				
			9 : 
				begin 
					LED_Binary <= 9'b100000000; 
					count = 0;
				end
				
			10 : 
				begin 
					LED_Binary <= 9'b001000001; 
					count = 0;
				end
				
			11 : 
				begin 
					LED_Binary <= 9'b001000010; 
					count = 0;
				end
				
			12 : 
				begin 
					LED_Binary <= 9'b001000011; 
					count = 0;
				end
				
			13 : 
				begin 
					LED_Binary <= 9'b001000100; 
					count = 0;
				end
				
			14 : 
				begin	
					case(count)
						0 : 
							begin
								LED_Binary <= 9'b000000000;
								
							end
						1 :  
							begin
								LED_Binary <= 9'b000000001;
								
							end
						2 :  
							begin
								LED_Binary <= 9'b000000010;
								
							end
						3 :  
							begin
								LED_Binary <= 9'b000000100;
								
							end
						4 :  
							begin
								LED_Binary <= 9'b000001000;
								
							end
						5 :  
							begin
								LED_Binary <= 9'b000010000;
								
							end
						6 :  
							begin
								LED_Binary <= 9'b000100000;
								
							end
						7 :  
							begin
								LED_Binary <= 9'b001000000;
								
							end
						8 :  
							begin
								LED_Binary <= 9'b010000000;
								
							end
						9 :  
							begin
								LED_Binary <= 9'b100000000;
								
							end
						default:
							begin
								LED_Binary <= 9'b000000000;
								
							end
					endcase
					count = count + 1;
					
					if(count == 10) begin
						count = 0;
					end
				end
				
				
			default: 
				begin
					LED_Binary <= 9'b000000000;
					count = 0;
				end
		endcase
	end
			
	
	/*
	begin LED_Binary = 8'b00000001; end
	begin LED_Binary = 8'b00000010; end
	begin LED_Binary = 8'b00000100; end
	begin LED_Binary = 8'b00001000; end
	begin LED_Binary = 8'b00010000; end
	begin LED_Binary = 8'b00100000; end
	begin LED_Binary = 8'b01000000; end
	begin LED_Binary = 8'b01000000; end
	begin LED_Binary = 8'b10000000; end
	begin LED_Binary = 8'b01000001; end
	begin LED_Binary = 8'b01000010; end
	begin LED_Binary = 8'b01000011; end
	begin LED_Binary = 8'b01000100; end
	begin LED_Binary = 8'b10000000; end
	begin LED_Binary = 8'b00000000; end
	*/
	
	/*LED_Fade_Top # (.FADE_TIME(4), .FADE_LEVELS(50), .AUTO(1), .CLOCKSPEED(10000000)) auto_fade
	(
		.sys_clk	(CLK),
		.RST_n		(RST_n),
		.CTRL		(1'b0),
		.BRIGHTNESS	(1'b1),
		.LED		(LED[4])
	);*/
		
	
endmodule